function ABlearn(iRun)

% <iRun>: the No. of the random run
% <Totalsketch>: allowed number of selected Gabors   

sx = 120; sy = 120;

% load natural statistics
load(sprintf('naturalStat_run%d',iRun));

% load the positive images
load(sprintf('../RandomSplits/split%d.mat',iRun),'posTrainImages');

% CONSTANTS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Totalsketch = 120;  % allowed number of selected local mean features   
gainThres = 1e-6;

% TRAINING EXAMPLES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ==================================================================
%  input images from a folder
% ==================================================================
posFolder = '../posAll';  % image input folder
nimage = length(posTrainImages);
Isize = zeros(nimage, 2);
I = cell(1,nimage);
for i = 1 : nimage
    im = imread([posFolder '\' posTrainImages{i}]);
    if size(im,3) == 3
        im = rgb2gray(im);
    end
    im = double(im);
    im = imresize(im,[sx sy]);
    I{i} = im;
    Isize(i, :) = size(I{i});
end

Sx = zeros(1, nimage) + sx; 
Sy = zeros(1, nimage) + sy;

% FILTERING IMAGES %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp('start filtering');
tic
[allfilter, allsymbol] = makefilter(scale, norient);  % generate Gabor filters 
h = (size(allfilter{1}, 1)-1)/2;  % half size of Gabor
allfiltered = applyfilterfftsame(I, allfilter);  % filter training images
disp(['filtering time: ' num2str(toc) ' seconds']);


% the following three lines are the only things that need to be added
mex Ctransform.c;
Ctransform(nimage, norient, allfiltered, Sx, Sy, Upperbound);

localMeanMap = cell(nimage,norient);

% compute local mean maps using (the integral image) fft 
averageFilter = ones(2*ax+1,2*ax+1)/(2*ax+1)/(2*ax+1);
for i = 1:nimage
    imsize = size(allfiltered{i,1});
    ave = zeros(imsize);
    for o = 1:norient
        localMeanMap{i,o} = filter2(averageFilter,allfiltered{i,o},'same');
        ave = ave + 1/norient * localMeanMap{i,o};
    end
    for o = 1:norient
        localMeanMap{i,o} = localMeanMap{i,o} ./ ( ave + 1e-10 );
    end
end


meanLocalMeanMap = zeros(sx,sy,norient);
for i = 1:nimage
    for o = 1:norient
        meanLocalMeanMap(:,:,o) = meanLocalMeanMap(:,:,o) + 1/nimage * localMeanMap{i,o};
    end
end

% VARIABLES FOR MEX-C %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Mi = zeros(1, Totalsketch);  % orientation of selected Gabor
Mx = zeros(1, Totalsketch);  % position of selected Gabor
My = zeros(1, Totalsketch); 
Mm = zeros(1, Totalsketch);  % mean
lambda = zeros(1, Totalsketch);  % lambda
logZ = zeros(1, Totalsketch);  % logZ
gain = zeros(Totalsketch, 1);  % coding gain for selected Gabor

tic

gainMap = zeros(sx,sy,norient);
ind = zeros(sx,sy,o);
for x = (h+ax)+1:sx-(h+ax)
    for y = 1+(h+ax):sy-(h+ax)
        for o = 1:norient
            [dif ind(x,y,o)] = min( abs(storedMean{o} - meanLocalMeanMap(x,y,o)) );
            ind(x,y,o) = min(length(storedMean{o}),max(1,ind(x,y,o)));
            gainMap(x,y,o) = storedGain{o}(ind(x,y,o));
        end
    end
end

inhibited = zeros(sx,sy,norient);

nF = Totalsketch;
for k = 1:nF
    [maxGain here] = max(gainMap(:) .* double(1-inhibited(:))  );
    binNo = ind(here);
    
    if maxGain <= gainThres
        nF = k-1;
        break
    end
    
    ori = ceil(here/sx/sy); % start from 1 !!
    Mi(k) = ori;
    col = ceil((here - (ori-1) * sx * sy)/sx);
    My(k) = col;
    row = here - (ori-1) * sx * sy - (col-1)*sx; 
    Mx(k) = row;
    
    gain(k) = maxGain;
    Mm(k) = storedMean{ori}(binNo);
    lambda(k) = storedLambda{ori}(binNo);
    logZ(k) = storedLogZ{ori}(binNo);
    
    % do inhibition
    left = max(1,col-ax-h);
    right = min(sy,col+ax+h);
    top = max(1,row-ax-h);
    bottom = min(sx,row+ax+h);
    for iOri = ori-1:ori+1
        iOri2 = iOri;
        if iOri < 1
            iOri2 = iOri + norient;
        end
        if iOri > norient
            iOri2 = iOri - norient;
        end
        inhibited(top:bottom,left:right,iOri2) = 1;
        gainMap(top:bottom,left:right,iOri2) = 0;
    end
end
Mm = Mm(1:nF); Mx = Mx(1:nF); My = My(1:nF);
gain = gain(1:nF); Mi = Mi(1:nF);
lambda = lambda(1:nF);
logZ = logZ(1:nF);

disp(['learning time: ' num2str(toc) ' seconds']);

save(sprintf('model_run%d.mat',iRun),...
    'ax','Mi','Mx','My', 'Mm','lambda','logZ','h','scale',...
    'norient','allfilter','allsymbol','Upperbound',...
    'Totalsketch','nF','M','lam','e','lz','gain','sx','sy');


